unit ClockClientMainForm;

interface

uses
  Clock_TLB,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TClockClientForm = class(TForm)
    btnSetAlarm: TButton;
    chkAlarmSet: TCheckBox;
    Timer1: TTimer;
    lblDateTime: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure btnSetAlarmClick(Sender: TObject);
  private
    FSinkCookie: Integer;
  public
    ClockServer: IApplication;
  end;

var
  ClockClientForm: TClockClientForm;

implementation

{$R *.DFM}

uses
  ComObj, ClockClientImpl;

procedure TClockClientForm.FormCreate(Sender: TObject);
const
  ProgID = 'Clock.Application';
begin
  try
    ClockServer := GetActiveOleObject(ProgID) as IApplication;
  except
    ClockServer := CreateOleObject(ProgID) as IApplication;
  end;
  //No need to keep hold of event sink. It will be destroyed
  //through interface reference counting when the client
  //disconnects from the server in the form's OnDestroy event handler
  InterfaceConnect(ClockServer, IApplicationEvents, TClockSink.Create, FSinkCookie)
end;

procedure TClockClientForm.FormDestroy(Sender: TObject);
begin
  InterfaceDisconnect(ClockServer, IApplicationEvents, FSinkCookie);
end;

procedure TClockClientForm.Timer1Timer(Sender: TObject);
begin
  try
    lblDateTime.Caption := DateTimeToStr(ClockServer.CurrentDateTime);
    chkAlarmSet.Checked := ClockServer.AlarmSet;
  except
    lblDateTime.Caption := 'Date/time server not available right now';
    chkAlarmSet.State := cbGrayed;
  end
end;

procedure TClockClientForm.btnSetAlarmClick(Sender: TObject);
begin
  if not ClockServer.AlarmSet then
    ClockServer.Alarm := Now + (5 / SecsPerDay)
  else
    ShowMessage('Alarm already set!')
end;

end.
